;Modified and Extended By Elias Tsiantas [25/6/03]
;Particle system based on simon@acid's single surface particle system:

;{
;simon@acid.co.nz
; demonstrates particle system using single surface of dynamic tris
; tris use position of particle and angle of camera to rebuild their
; billboard every update facing the camera with only 3 calls to vertexcoords
; animation of particles is possible by scrolling through frames in texture 
;}


;This one is based on tris and is ultra fast.




;##########################################################################




;point emitter
Type emitter

Field hndl.dirt;handle

Field id%;id count

;Field parent%


;creation fields
Field tris%;how many tris for this emitter
Field tex$;the texture name
Field texflag%,fx%,blend%,alpha#;texture flag,fx,blend,alpha
Field vrange#,vis;if camera>vrange then hide the emitter
Field r%,g%,b%;r,g,b colors [works only when use vertex color is disabled in fx]
Field gravity#;gravity
Field updtrate%;every how many cycles to update the particles [0=every cycle]
;rotation
Field rflag%;flag to rotate the emitter [0=no rotate,1=xy,2=xz,3=yz,4=xyz rotation]
Field rcount#,rate#;rcount=0-360 counter / rate#=how fast to rotate
;position
Field fact1#,fact2#;sinusoidal position of emitter factors
;add fields
Field size#,x#,y#,z#;size of particle / position of emitter
Field vsize#,vx1#,vx2#,vy1#,vy2#,vz1#,vz2#;velocity sizes
Field rgb,cycles;rgb=hex value for vertex coloring, cycles=time to live for every particle

Field emitterlife%;emitter life in (t+millisecs) , if =0 then runs forever

;periodical adding of particles
;adf=0=always add, adf=1=add sinusoidal , adf=2=add random sinusoidal
;adc#=counter, adr1=addrate1[, adr2=addrate2 / only when random is used]
Field adf,adc#,adr1#,adr2#

End Type



;point type emitter type

Type dirt
	Field	id
	Field	mesh
	Field   x#,y#,z#
	Field	surf

	Field	cache.adirt
	Field	pos.adirt
	
	Field	texture
	Field	dirtx%;texture cycle
	
	Field   ctim;millisecs counter
	Field   tim;every how many millisecs it will be updated
	
	
End Type



;==========================================================================



;plane/linear emitter [same as above only with different behavior]
Type pemit

Field id%

Field hndl.pirt

;creation fields
Field tris%
Field tex$
Field texflag%,fx%,blend%,alpha#
Field vrange#,vis
Field r%,g%,b%
Field gravity#
Field updtrate%

Field align%;<-----------align flag to face the camera or not
Field ax1#,ay1#,az1#,ax2#,ay2#,az2#

;add fields
Field size#,x1#,x2#,y1#,y2#,z1#,z2#;<use these ranges to make a plane type emitter [see examples]

Field vsize#,vx1#,vx2#,vy1#,vy2#,vz1#,vz2#
Field rgb,cycles

Field emitterlife%

;periodical adding of particles
;adf=0=always add, adf=1=add sinusoidal , adf=2=add random sinusoidal
;adc#=counter, adr1=addrate1[, adr2=addrate2 / only when random is used]
Field adf,adc#,adr1#,adr2#


End Type





;plane type emitter type

Type pirt
	Field	id
	Field	mesh
	Field   x#,y#,z#
	Field	surf

	Field	cache.adirt
	Field	pos.adirt
	
	Field	texture
	Field	dirtx%;texture cycle?
	
	Field   ctim;millisecs counter
	Field   tim;every how many millisecs it will be updated
	
;{	
	Field   align%;0=align to camera , 1=align to the below vectors
	Field   ax1#,ay1#,az1#;if align=1 then align to these vectors
	Field   ax2#,ay2#,az2#;	-"-
	
			;1,0,0
			;0,1,0;->xy plane
			
			;1,0,0
			;0,0,1;->xz plane
			
			;0,1,0
			;0,0,1;->yz plane
								;}
End Type


;==========================================================================



;COMMON

Type vector
	Field id%
	Field	x#
	Field	y#
	Field	z#
End Type



Type adirt
	Field	id
	Field	link.adirt
	Field	index
	Field	time
	Field	rgb
	Field	pos.vector
	Field	vel.vector
	Field	size#,vsize#
	
	Field   grav#;gravity
	
End Type






;##########################################################################




;Create the emitters
Function PH_make_emitters()

For em.emitter=Each emitter
em\hndl=CreateDirt(em\id,em\x,em\y,em\z,em\tris,em\tex$,em\texflag,em\fx,em\blend,em\alpha,em\vrange,em\r,em\g,em\b,em\gravity,em\updtrate)
Next

For pm.pemit=Each pemit
pm\hndl=CreateDirt2(pm\id,(pm\x1+pm\x2)/2.0,(pm\y1+pm\y2)/2.0,(pm\z1+pm\z2)/2.0,pm\tris,pm\tex$,pm\texflag,pm\fx,pm\blend,pm\alpha,pm\vrange,pm\r,pm\g,pm\b,pm\gravity,pm\updtrate,pm\align,pm\ax1,pm\ay1,pm\az1,pm\ax2,pm\ay2,pm\az2)
Next

End Function




;Add particles update routine
Function PH_add_particles()


For em.emitter=Each emitter

x#=EntityX(em\hndl\mesh)
y#=EntityY(em\hndl\mesh)
z#=EntityZ(em\hndl\mesh)


;update always
If em\adf=0 Goto update

;update periodical-sinusoidal
If em\adf=1
em\adc#=em\adc#+em\adr1#;em\adt#=em\adt#+RND(em\adr1#,em\adr2#)
If em\adc#>=(360-em\adr1#) em\adc#=0
If Sin(em\adc#)>0
Goto update
Else
Goto not_update
EndIf
EndIf


;update random-sinusoidal
If em\adf=2
updtrf#=Rnd(em\adr1#,em\adr2#)
em\adc#=em\adc#+updtrf#
If em\adc#>=(360-updtrf#) em\adc#=0
If Sin(em\adc#)>0
Goto update
Else
Goto not_update
EndIf
EndIf





.update

Select em\rflag

Case 0;normal
AddDirt(em\hndl,em\size,x,y,z,em\vsize,Rnd(em\vx1,em\vx2),Rnd(em\vy1,em\vy2),Rnd(em\vz1,em\vz2),em\rgb,em\cycles)



Case 1;xy  rotation
em\rcount=em\rcount+em\rate
If em\rcount>=(360-em\rate) em\rcount=0
AddDirt(em\hndl,em\size,x,y,z,em\vsize,Rnd(em\vx1*Sin(em\rcount),em\vx2*Sin(em\rcount)),Rnd(em\vy1*Cos(em\rcount),em\vy2*Cos(em\rcount)),Rnd(em\vz1,em\vz2),em\rgb,em\cycles)
Case 2;xz  rotation
em\rcount=em\rcount+em\rate
If em\rcount>=(360-em\rate) em\rcount=0
AddDirt(em\hndl,em\size,x,y,z,em\vsize,Rnd(em\vx1*Sin(em\rcount),em\vx2*Sin(em\rcount)),Rnd(em\vy1,em\vy2),Rnd(em\vz1*Cos(em\rcount),em\vz2*Cos(em\rcount)),em\rgb,em\cycles)
Case 3;yz  rotation
em\rcount=em\rcount+em\rate
If em\rcount>=(360-em\rate) em\rcount=0
AddDirt(em\hndl,em\size,x,y,z,em\vsize,Rnd(em\vx1,em\vx2),Rnd(em\vy1*Cos(em\rcount),em\vy2*Cos(em\rcount)),Rnd(em\vz1*Sin(em\rcount),em\vz2*Sin(em\rcount)),em\rgb,em\cycles)
Case 4;xyz
em\rcount=em\rcount+em\rate
If em\rcount>=(360-em\rate) em\rcount=0
AddDirt(em\hndl,em\size,x,y,z,em\vsize,Rnd(em\vx1*Cos(em\rcount),em\vx2*Cos(em\rcount)),Rnd(em\vy1*Cos(em\rcount),em\vy2*Cos(em\rcount)),Rnd(em\vz1*Cos(em\rcount),em\vz2*Cos(em\rcount)),em\rgb,em\cycles)



Case 5;xy position
em\rcount=em\rcount+em\rate
If em\rcount>=(360-em\rate) em\rcount=0
AddDirt(em\hndl,em\size,x+em\fact1*Sin(em\rcount),y+em\fact2*Cos(em\rcount),z,em\vsize,Rnd(em\vx1,em\vx2),Rnd(em\vy1,em\vy2),Rnd(em\vz1,em\vz2),em\rgb,em\cycles)
Case 6;xz position
em\rcount=em\rcount+em\rate
If em\rcount>=(360-em\rate) em\rcount=0
AddDirt(em\hndl,em\size,x+em\fact1*Sin(em\rcount),y,z+em\fact2*Cos(em\rcount),em\vsize,Rnd(em\vx1,em\vx2),Rnd(em\vy1,em\vy2),Rnd(em\vz1,em\vz2),em\rgb,em\cycles)
Case 7;yz position
em\rcount=em\rcount+em\rate
If em\rcount>=(360-em\rate) em\rcount=0
AddDirt(em\hndl,em\size,x,y+em\fact1*Sin(em\rcount),z+em\fact2*Cos(em\rcount),em\vsize,Rnd(em\vx1,em\vx2),Rnd(em\vy1,em\vy2),Rnd(em\vz1,em\vz2),em\rgb,em\cycles)
Case 8;yz position
em\rcount=em\rcount+em\rate
If em\rcount>=(360-em\rate) em\rcount=0
AddDirt(em\hndl,em\size,x+em\fact1*Sin(em\rcount),y+em\fact2*Cos(em\rcount),z+em\fact1*Sin(em\rcount),em\vsize,Rnd(em\vx1,em\vx2),Rnd(em\vy1,em\vy2),Rnd(em\vz1,em\vz2),em\rgb,em\cycles)
End Select


.not_update

Next


;---



For pm.pemit=Each pemit


;update always
If pm\adf=0 Goto p_update

;update periodical-sinusoidal
If pm\adf=1
pm\adc#=pm\adc#+pm\adr1#;
If pm\adc#>=(360-pm\adr1#) pm\adc#=0
If Sin(pm\adc#)>0
Goto p_update
Else
Goto p_not_update
EndIf
EndIf


;update random-sinusoidal
If pm\adf=2
updtrf#=Rnd(pm\adr1#,pm\adr2#)
pm\adc#=pm\adc#+updtrf#
If pm\adc#>=(360-updtrf#) pm\adc#=0
If Sin(pm\adc#)>0
Goto p_update
Else
Goto p_not_update
EndIf
EndIf

.p_update

x#=EntityX(pm\hndl\mesh)
y#=EntityY(pm\hndl\mesh)
z#=EntityZ(pm\hndl\mesh)

AddDirt2(pm\hndl,pm\size,x+Rnd(pm\x1,pm\x2),y+Rnd(pm\y1,pm\y2),z+Rnd(pm\z1,pm\z2),pm\vsize,Rnd(pm\vx1,pm\vx2),Rnd(pm\vy1,pm\vy2),Rnd(pm\vz1,pm\vz2),pm\rgb,pm\cycles)
Next

.p_not_update


End Function







;Update particles routine
Function PH_update_particles(cam)

cx#=EntityX(cam):cy#=EntityY(cam):cz#=EntityZ(cam)



	For em.emitter=Each emitter
	
	UpdateDirt ( em\hndl,cam);,em\parent )
	
	;hide/show based on distance from the camera
	vx#=cx#-em\x:vy#=cy#-em\y:vz#=cz#-em\z
	dist#=Sqr(vx#*vx#+vy#*vy#+vz#*vz#)
	If dist#<em\vrange
	If em\vis=1 ShowEntity em\hndl\mesh:em\vis=0
	Else
	If em\vis=0 HideEntity em\hndl\mesh:em\vis=1
	EndIf
	
	;check for limited life emitters
	If em\emitterlife<>0
	If em\emitterlife<MilliSecs() Then DestroyDirt(em\id,0)
	EndIf
		
	Next


	For pm.pemit=Each pemit
	UpdateDirt2 ( pm\hndl,cam )

	;hide/show based on distance from the camera
	vx#=cx#-pm\hndl\x:vy#=cy#-pm\hndl\y:vz#=cz#-pm\hndl\z
	dist#=Sqr(vx#*vx#+vy#*vy#+vz#*vz#)
	If dist#<pm\vrange
	If pm\vis=1 ShowEntity pm\hndl\mesh:pm\vis=0
	Else
	If pm\vis=0 HideEntity pm\hndl\mesh:pm\vis=1
	EndIf

	;check for limited life emitters
	If pm\emitterlife<>0
	If pm\emitterlife<MilliSecs() Then DestroyDirt(pm\id,1)
	EndIf
		
	Next
	
End Function





Function PH_Emitters_alive()

For em.emitter=Each emitter
emal=emal+1
Next
For pm.pemit=Each pemit
emal=emal+1
Next
Return emal

End Function



Function PH_Emitters_hidden()

For em.emitter=Each emitter
If em\vis=1 emal=emal+1
Next
For pm.pemit=Each pemit
If pm\vis=1 emal=emal+1
Next
Return emal

End Function


Function PH_Emitters_shown()

For em.emitter=Each emitter
If em\vis=0 emal=emal+1
Next
For pm.pemit=Each pemit
If pm\vis=0 emal=emal+1
Next
Return emal

End Function





;##########################################################################
;
;POINT TYPE EMITTER FUNCTIONS
;
;##########################################################################


Function CreateDirt.dirt(id,x#,y#,z#,num,tex$,texflag,fx,blend,alpha#,vrange#, r,g,b ,gravity#,tupdate);

If num>21845 Then num=21845;max tri num

	brush=CreateBrush()	
	dust=LoadTexture(tex$,texflag);
	BrushTexture brush,dust
	BrushFX brush,fx
	BrushBlend brush,blend;
	BrushAlpha brush,alpha#
	BrushColor brush,r,g,b
	

	d.dirt=New dirt
	
	d\id=id
	d\texture=dust
	
	d\tim=tupdate
	
	d\mesh=CreateMesh()
	
	PositionEntity d\mesh,x/2,y/2,z/2,1
	d\x=x:d\y=y:d\z=z

	
	d\surf=CreateSurface(d\mesh,brush)
	
	
	For i=1 To num
		MakeDirt(d,gravity#,id,d\x,d\y,d\z)
	Next
	

	FreeBrush brush
	;FreeTexture dust & d\texture
	
	Return d
	
End Function







Function MakeDirt(d.dirt,gravity#,id,x#,y#,z#)
	a.adirt=New adirt
	a\link=d\cache
	d\cache=a
	s=d\surf
	
	a\id=id

	i=AddVertex(s,x,y,z ,0.5,-.4)
	  AddVertex(s,x,y,z   ,1.4,1)
	  AddVertex(s,x,y,z   ,-.4,1)
	AddTriangle s,i,i+1,i+2

	a\index=i
	a\time=0

	a\pos=New vector
	a\pos\id=id
	a\vel=New vector
	a\vel\id=id

	a\grav=gravity
	
End Function







Function AddDirt(s.dirt,size#,x#,y#,z#,vsize#,vx#,vy#,vz#,rgb,t)

	a.adirt=s\pos
	If a=Null a=s\cache 
	s\pos=a\link
	
	a\pos\x=x
	a\pos\y=y
	a\pos\z=z

	a\vel\x=vx
	a\vel\y=vy
	a\vel\z=vz
	
	a\rgb=rgb
	r=(rgb Shr 16) And 255
	g=(rgb Shr 8) And 255
	b=rgb And 255
	surf=s\surf
	i=a\index
	VertexColor surf,i+0,r,g,b
	VertexColor surf,i+1,r,g,b 
	VertexColor surf,i+2,r,g,b 
	
	a\time=t
	a\size=size
	a\vsize=vsize
	
End Function







Function UpdateDirt(d.dirt,cam);,parent)
	
	If d\ctim<MilliSecs()
	
	;d\dirtx=d\dirtx+1;texture cycle
	
	surf=d\surf
	a.adirt=d\cache
	TFormVector -1,0,0,cam,d\mesh
	camxx#=TFormedX()
	camxy#=TFormedY()
	camxz#=TFormedZ()
	TFormVector 0,-1,0,cam,d\mesh
	camyx#=TFormedX()
	camyy#=TFormedY()
	camyz#=TFormedZ()
	While a<>Null
		If a\time>0
			a\time=a\time-1
			a\vel\y=a\vel\y-a\grav
		
;			AddVector a\pos,a\vel
			p.vector=a\pos
			
			p\x=p\x+a\vel\x
			p\y=p\y+a\vel\y
			p\z=p\z+a\vel\z	
			

			a\size=a\size+a\vsize

			
			If a\time=0 ;Or a\pos\y<0;uncomment this to hide dirt below 0 or whatever
				HideDirt(d,a)
			Else
				scale#=a\size
				cxx#=camxx*scale
				cxy#=camxy*scale
				cxz#=camxz*scale
				cyx#=camyx*scale
				cyy#=camyy*scale
				cyz#=camyz*scale
				i=a\index
			
				x#=p\x-cyx
				y#=p\y-cyy
				z#=p\z-cyz
				VertexCoords surf,i,x,y,z
				x=x+cyx-cxx
				y=y+cyy-cxy
				z=z+cyz-cxz
				VertexCoords surf,i+1,x,y,z
				x=x+cxx+cxx
				y=y+cxy+cxy
				z=z+cxz+cxz
				VertexCoords surf,i+2,x,y,z

			EndIf
		EndIf
		a=a\link
	Wend
	
	d\ctim=d\tim+MilliSecs()
	EndIf
	
End Function






Function HideDirt(d.dirt,a.adirt)
	surf=d\surf
	i=a\index

x#=EntityX(d\mesh)
y#=EntityY(d\mesh)
z#=EntityZ(d\mesh)

	VertexCoords surf,i,  x,y,z
	VertexCoords surf,i+1,x,y,z
	VertexCoords surf,i+2,x,y,z

	a\time=0
End Function





;##########################################################################
;
;PLANE TYPE EMITTER FUNCTIONS
;
;##########################################################################


Function CreateDirt2.pirt(id,x#,y#,z#,num,tex$,texflag,fx,blend,alpha#,vrange#, r,g,b ,gravity#,tupdate,align=0,ax1=0,ay1=0,az1=0,ax2=0,ay2=0,az2=0);

If num>21845 Then num=21845;max tri num

	brush=CreateBrush()	
	dust=LoadTexture(tex$,texflag);
	BrushTexture brush,dust
	BrushFX brush,fx
	BrushBlend brush,blend;
	BrushAlpha brush,alpha#
	BrushColor brush,r,g,b
	

	pl.pirt=New pirt
	
	pl\id=id
	pl\texture=dust
	
	pl\tim=tupdate
	
	pl\mesh=CreateMesh()
	pl\x=x:pl\y=y:pl\z=z
	
	pl\surf=CreateSurface(pl\mesh,brush)
	
	pl\align=align
	
	pl\ax1=ax1:pl\ay1=ay1:pl\az1=az1
	pl\ax2=ax2:pl\ay2=ay2:pl\az2=az2

	
	For i=1 To num
		MakeDirt2(pl,gravity#,id,x#,y#,z#)
	Next

	FreeBrush brush
	;FreeTexture dust & p\texture

	
	Return pl
	
End Function






Function MakeDirt2(pl.pirt,gravity#,id,x#,y#,z#)
	a.adirt=New adirt
	a\link=pl\cache
	pl\cache=a
	s=pl\surf
	
	a\id=id

	i=AddVertex(s,x,y,z ,0.5,-.4)
	  AddVertex(s,x,y,z   ,1.4,1)
	  AddVertex(s,x,y,z   ,-.4,1)
	AddTriangle s,i,i+1,i+2

	a\index=i
	a\time=0
	
	a\pos=New vector
	a\pos\id=id
	a\vel=New vector
	a\vel\id=id
	
	a\grav=gravity
	
End Function








Function AddDirt2(s.pirt,size#,x#,y#,z#,vsize#,vx#,vy#,vz#,rgb,t)

	a.adirt=s\pos
	If a=Null a=s\cache 
	s\pos=a\link
	
	a\pos\x=x
	a\pos\y=y
	a\pos\z=z

	a\vel\x=vx
	a\vel\y=vy
	a\vel\z=vz
	
	a\rgb=rgb
	r=(rgb Shr 16) And 255
	g=(rgb Shr 8) And 255
	b=rgb And 255
	surf=s\surf
	i=a\index
	VertexColor surf,i+0,r,g,b
	VertexColor surf,i+1,r,g,b 
	VertexColor surf,i+2,r,g,b 
	
	a\time=t
	a\size=size
	a\vsize=vsize

	
End Function






Function UpdateDirt2(pl.pirt,cam)
	
	If pl\ctim<MilliSecs()
	
	;p\dirtx=p\dirtx+1;texture cycle
	
	surf=pl\surf
	a.adirt=pl\cache
	
	If pl\align=0;if aligned to camera
	
	TFormVector -1,0,0,cam,0
	camxx#=TFormedX()
	camxy#=TFormedY()
	camxz#=TFormedZ()
	TFormVector 0,-1,0,cam,0
	camyx#=TFormedX()
	camyy#=TFormedY()
	camyz#=TFormedZ()

	
	While a<>Null
		If a\time>0
			a\time=a\time-1
			a\vel\y=a\vel\y-a\grav
		
;			AddVector a\pos,a\vel
			p.vector=a\pos
			p\x=p\x+a\vel\x
			p\y=p\y+a\vel\y
			p\z=p\z+a\vel\z
			a\size=a\size+a\vsize
			
			If a\time=0
				HideDirt2(pl,a)
			Else
				scale#=a\size
				cxx#=camxx*scale
				cxy#=camxy*scale
				cxz#=camxz*scale
				cyx#=camyx*scale
				cyy#=camyy*scale
				cyz#=camyz*scale
				i=a\index
			
				x#=p\x-cyx
				y#=p\y-cyy
				z#=p\z-cyz
				VertexCoords surf,i,x,y,z
				x=x+cyx-cxx
				y=y+cyy-cxy
				z=z+cyz-cxz
				VertexCoords surf,i+1,x,y,z
				x=x+cxx+cxx
				y=y+cxy+cxy
				z=z+cxz+cxz
				VertexCoords surf,i+2,x,y,z
				
				

			EndIf
		EndIf
		a=a\link
	Wend

	Else;else align to xyz plane

	TFormVector pl\ax1,pl\ay1,pl\az1,0,0
	camxx#=TFormedX():camxy#=TFormedY():camxz#=TFormedZ()
	TFormVector pl\ax2,pl\ay2,pl\az2,0,0
	camyx#=TFormedX():camyy#=TFormedY():camyz#=TFormedZ()	
	
	While a<>Null
		If a\time>0
			a\time=a\time-1
			a\vel\y=a\vel\y-a\grav
		
;			AddVector a\pos,a\vel
			p.vector=a\pos
			p\x=p\x+a\vel\x
			p\y=p\y+a\vel\y
			p\z=p\z+a\vel\z
			a\size=a\size+a\vsize
			
			If a\time=0
				HideDirt2(pl,a)
			Else
				scale#=a\size
				cxx#=camxx*scale
				cxy#=camxy*scale
				cxz#=camxz*scale
				cyx#=camyx*scale
				cyy#=camyy*scale
				cyz#=camyz*scale
				i=a\index
			
				x#=p\x-cyx
				y#=p\y-cyy
				z#=p\z-cyz
				VertexCoords surf,i,x,y,z
				x=x+cyx-cxx
				y=y+cyy-cxy
				z=z+cyz-cxz
				VertexCoords surf,i+1,x,y,z
				x=x+cxx+cxx
				y=y+cxy+cxy
				z=z+cxz+cxz
				VertexCoords surf,i+2,x,y,z
				
			EndIf
		EndIf
		a=a\link
	Wend
		
	EndIf
		
	pl\ctim=pl\tim+MilliSecs()
	EndIf
	
End Function






Function HideDirt2(pl.pirt,a.adirt)
	surf=pl\surf
	i=a\index

x#=EntityX(pl\mesh)
y#=EntityY(pl\mesh)
z#=EntityZ(pl\mesh)

	VertexCoords surf,i,  x,y,z
	VertexCoords surf,i+1,x,y,z
	VertexCoords surf,i+2,x,y,z

	a\time=0
End Function





;##########################################################################





Function DestroyDirt(id,flag)

Select flag

Case 0;point emitter

em.emitter=First emitter
While Not em\id = id:em=After em:Wend
Delete em

d.dirt=First dirt
While Not d\id = id:d=After d:Wend
FreeEntity d\mesh
Delete d

Case 1;plane emitter

pm.pemit=First pemit
While Not pm\id = id:pm=After pm:Wend
Delete pm

pl.pirt=First pirt
While Not pl\id = id:pl=After pl:Wend
FreeEntity pl\mesh
Delete pl

End Select

a.adirt=First adirt
For a.adirt=Each adirt
If a\id=id Then Delete a
Next

For p.vector=Each vector
If p\id=id Then Delete p
Next

End Function








Function Destroy_all_particles()

For em.emitter=Each emitter
Delete em
Next

For pm.pemit=Each pemit
Delete pm
Next

For d.dirt=Each dirt
FreeEntity d\mesh
Delete d
Next

For pl.pirt=Each pirt
FreeEntity pl\mesh
Delete pl
Next

Delete Each adirt

Delete Each vector

End Function
